import type { StrictUnion } from 'simplytyped';
/** If the variable `GITHUB_API_URL` or `GITHUB_API` exists, use that, otherwise use the value `https://api.github.com`. */
declare type GitHubApiUrl = StrictUnion<{
    /** https://docs.github.com/en/free-pro-team@latest/actions/reference/environment-variables#default-environment-variables */
    GITHUB_API_URL: string;
} | {
    /** https://github.com/search?l=JavaScript&q=GITHUB_API&type=Code */
    GITHUB_API: string;
}>;
/**
 * If the variable `GITHUB_ACCESS_TOKEN` or `GITHUB_TOKEN` exists, use that according to:
 * https://developer.github.com/v3/#oauth2-token-sent-as-a-parameter
 */
declare type GitHubToken = StrictUnion<{
    /** https://github.com/search?q=GITHUB_ACCESS_TOKEN&type=code */
    GITHUB_ACCESS_TOKEN: string;
} | {
    /** https://docs.github.com/en/free-pro-team@latest/actions/reference/authentication-in-a-workflow#about-the-github_token-secret */
    GITHUB_TOKEN: string;
}>;
/**
 * If the variables `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET` exist, use that according to:
 * https://developer.github.com/v3/#oauth2-keysecret
 */
interface GitHubClient {
    GITHUB_CLIENT_ID: string;
    GITHUB_CLIENT_SECRET: string;
}
/**
 * Provide `GITHUB_ACCESS_TOKEN` or `GITHUB_TOKEN`,
 * or a combination of `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET`,
 * optionally with `GITHUB_ACCESS_TOKEN` or `GITHUB_TOKEN`.
 * https://developer.github.com/v3/#authentication
 */
export declare type GitHubCredentials = StrictUnion<GitHubToken | GitHubClient> & Partial<GitHubApiUrl>;
/**
 * Check whether or not sufficient GitHub credentials were supplied.
 * @returns `true` if valid
 * @throws if invalid
 */
export declare function validate(credentials: GitHubCredentials): boolean;
/**
 * Get the desired GitHub Access Token from the credentials.
 * You probably want {@link getURL} instead.
 */
export declare function getAccessToken(credentials: GitHubCredentials): string | null;
/**
 * Get the GitHub Authorization Search Params.
 * You probably want {@link getURL} instead.
 * @param credentials The params to use for the authorization variables.
 * @param params If you wish to set the params on an existing URLSearchParams instance, then provide it here.
 * @returns The search params that you should append to your github API request url.
 * @throws If no valid GitHub Authorization was provided.
 */
export declare function getSearchParams(credentials: GitHubCredentials, params?: URLSearchParams): URLSearchParams;
/**
 * Get the GitHub Authorization as a Query String.
 * You probably want {@link getURL} instead.
 */
export declare function getQueryString(credentials: GitHubCredentials): string;
/**
 * Get the GitHub Authorization Header.
 * Use as the `Authorization` header within {@link fetch} calls.
 * You probably want {@link getHeaders} instead.
 */
export declare function getAuthHeader(credentials: GitHubCredentials): string;
/**
 * Get the headers to attach to the request to the GitHub API.
 * Use as the headers object within {@link fetch} calls.
 * Make sure to use with {@link getApiUrl} to make sure you are using the desired hostname.
 */
export declare function getHeaders(credentials: GitHubCredentials): {
    Accept: string;
    Authorization: string;
};
/**
 * Get the desired Github API URL string.
 * As this does not include any credentials, use with {@link getAuthHeader} to authorize correctly.
 * Otherwise use {@link getURL} to get a credentialed URL.
 */
export declare function getApiUrl(credentials: GitHubCredentials): string;
/**
 * Get the credentialed GitHub API URL instance.
 * Uses {@link getApiUrl} to fill the hostname, and uses {@link getSearchParams} to fill the credentials.
 */
export declare function getURL(credentials: GitHubCredentials, props?: {
    pathname?: string;
    searchParams?: URLSearchParams;
}): URL;
/**
 * Get the credentialed GitHub API URL string from {@link getURL}.
 */
export declare function getUrl(credentials: GitHubCredentials, props?: {
    pathname?: string;
    searchParams?: URLSearchParams;
}): string;
/**
 * Redact any GitHub Credentials from a URL string.
 * @param value The string to redact credentials from.
 * @returns The string with the credentials redacted.
 */
export declare function redactSearchParams(value: string): string;
export {};
//# sourceMappingURL=index.d.ts.map