"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.redactSearchParams = exports.getUrl = exports.getURL = exports.getApiUrl = exports.getHeaders = exports.getAuthHeader = exports.getQueryString = exports.getSearchParams = exports.getAccessToken = exports.validate = void 0;
/**
 * Check whether or not sufficient GitHub credentials were supplied.
 * @returns `true` if valid
 * @throws if invalid
 */
function validate(credentials) {
    const accessToken = getAccessToken(credentials);
    if (accessToken ||
        (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET)) {
        return true;
    }
    else {
        throw new Error('missing github credentials; provide `GITHUB_ACCESS_TOKEN` or `GITHUB_TOKEN`, or a combination of `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET`');
    }
}
exports.validate = validate;
/**
 * Get the desired GitHub Access Token from the credentials.
 * You probably want {@link getURL} instead.
 */
function getAccessToken(credentials) {
    return credentials.GITHUB_ACCESS_TOKEN || credentials.GITHUB_TOKEN || null;
}
exports.getAccessToken = getAccessToken;
/**
 * Get the GitHub Authorization Search Params.
 * You probably want {@link getURL} instead.
 * @param credentials The params to use for the authorization variables.
 * @param params If you wish to set the params on an existing URLSearchParams instance, then provide it here.
 * @returns The search params that you should append to your github API request url.
 * @throws If no valid GitHub Authorization was provided.
 */
function getSearchParams(credentials, params = new URLSearchParams()) {
    const accessToken = getAccessToken(credentials);
    if (accessToken) {
        params.set('access_token', accessToken);
    }
    else if (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET) {
        params.set('client_id', credentials.GITHUB_CLIENT_ID);
        params.set('client_secret', credentials.GITHUB_CLIENT_SECRET);
    }
    else {
        // throw with detail errors
        validate(credentials);
        // if that doesn't throw, then fallback to this
        throw new Error('invalid github credentials');
    }
    return params;
}
exports.getSearchParams = getSearchParams;
/**
 * Get the GitHub Authorization as a Query String.
 * You probably want {@link getURL} instead.
 */
function getQueryString(credentials) {
    return getSearchParams(credentials).toString();
}
exports.getQueryString = getQueryString;
/**
 * Get the GitHub Authorization Header.
 * Use as the `Authorization` header within {@link fetch} calls.
 * You probably want {@link getHeaders} instead.
 */
function getAuthHeader(credentials) {
    const accessToken = getAccessToken(credentials);
    if (accessToken) {
        return `token ${accessToken}`;
    }
    else if (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET) {
        return `Basic ${credentials.GITHUB_CLIENT_ID}:${credentials.GITHUB_CLIENT_SECRET}`;
    }
    else {
        throw new Error('missing github credentials for authorization header');
    }
}
exports.getAuthHeader = getAuthHeader;
/**
 * Get the headers to attach to the request to the GitHub API.
 * Use as the headers object within {@link fetch} calls.
 * Make sure to use with {@link getApiUrl} to make sure you are using the desired hostname.
 */
function getHeaders(credentials) {
    return {
        Accept: 'application/vnd.github.v3+json',
        Authorization: getAuthHeader(credentials),
    };
}
exports.getHeaders = getHeaders;
/**
 * Get the desired Github API URL string.
 * As this does not include any credentials, use with {@link getAuthHeader} to authorize correctly.
 * Otherwise use {@link getURL} to get a credentialed URL.
 */
function getApiUrl(credentials) {
    return (credentials.GITHUB_API_URL ||
        credentials.GITHUB_API ||
        'https://api.github.com');
}
exports.getApiUrl = getApiUrl;
/**
 * Get the credentialed GitHub API URL instance.
 * Uses {@link getApiUrl} to fill the hostname, and uses {@link getSearchParams} to fill the credentials.
 */
function getURL(credentials, props) {
    // fetch url
    const url = new URL(getApiUrl(credentials));
    // apply params
    getSearchParams(credentials, url.searchParams);
    if (props === null || props === void 0 ? void 0 : props.searchParams)
        props.searchParams.forEach((value, key) => url.searchParams.set(key, value));
    // apply pathname via append, as otherwise urls like `https://bevry.me/api/github` will not work
    if (props === null || props === void 0 ? void 0 : props.pathname)
        url.pathname += props.pathname;
    // return
    return url;
}
exports.getURL = getURL;
/**
 * Get the credentialed GitHub API URL string from {@link getURL}.
 */
function getUrl(credentials, props) {
    return getURL(credentials, props).toString();
}
exports.getUrl = getUrl;
/**
 * Redact any GitHub Credentials from a URL string.
 * @param value The string to redact credentials from.
 * @returns The string with the credentials redacted.
 */
function redactSearchParams(value) {
    return value.replace(/(&?)(access_token|client_id|client_secret)=\w+/gi, '$1$2=REDACTED');
}
exports.redactSearchParams = redactSearchParams;
